@@ -1,4 +1,4 @@
-# $Id: __init__.py 6433 2010-09-28 08:21:25Z milde $
+# $Id: __init__.py 7538 2012-11-23 01:18:49Z milde $
 # Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
@@ -11,11 +11,15 @@ This package contains modules for language-dependent features of Docutils.
 
 __docformat__ = 'reStructuredText'
 
+import sys
+
 from docutils.utils import normalize_language_tag
+if sys.version_info < (2,5):
+    from docutils._compat import __import__
 
 _languages = {}
 
-def get_language(language_code, reporter):
+def get_language(language_code, reporter=None):
     """Return module with language localizations.
 
     `language_code` is a "BCP 47" language tag.
@@ -23,17 +27,22 @@ def get_language(language_code, reporter):
     """
     # TODO: use a dummy module returning emtpy strings?, configurable?
     for tag in normalize_language_tag(language_code):
+        tag = tag.replace('-','_') # '-' not valid in module names
         if tag in _languages:
             return _languages[tag]
         try:
-            module = __import__(tag, globals(), locals())
+            module = __import__(tag, globals(), locals(), level=0)
         except ImportError:
-            continue
+            try:
+                module = __import__(tag, globals(), locals(), level=1)
+            except ImportError:
+                continue
         _languages[tag] = module
         return module
-    reporter.warning(
-        'language "%s" not supported: ' % language_code +
-        'Docutils-generated text will be in English.')
-    module = __import__('en', globals(), locals())
+    if reporter is not None:
+        reporter.warning(
+            'language "%s" not supported: ' % language_code +
+            'Docutils-generated text will be in English.')
+    module = __import__('en', globals(), locals(), level=1)
     _languages[tag] = module # warn only one time!
     return module
