@@ -1,4 +1,4 @@
-# $Id: __init__.py 4667 2006-07-12 21:40:56Z wiemann $
+# $Id: __init__.py 7486 2012-07-11 12:25:14Z milde $
 # Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
@@ -10,13 +10,18 @@ __docformat__ = 'reStructuredText'
 
 import re
 import codecs
+import sys
+
 from docutils import nodes
 from docutils.parsers.rst.languages import en as _fallback_language_module
+if sys.version_info < (2,5):
+    from docutils._compat import __import__
 
 
 _directive_registry = {
       'attention': ('admonitions', 'Attention'),
       'caution': ('admonitions', 'Caution'),
+      'code': ('body', 'CodeBlock'),
       'danger': ('admonitions', 'Danger'),
       'error': ('admonitions', 'Error'),
       'important': ('admonitions', 'Important'),
@@ -29,6 +34,7 @@ _directive_registry = {
       'topic': ('body', 'Topic'),
       'line-block': ('body', 'LineBlock'),
       'parsed-literal': ('body', 'ParsedLiteral'),
+      'math': ('body', 'MathBlock'),
       'rubric': ('body', 'Rubric'),
       'epigraph': ('body', 'Epigraph'),
       'highlights': ('body', 'Highlights'),
@@ -76,7 +82,7 @@ def directive(directive_name, language_module, document):
     normname = directive_name.lower()
     messages = []
     msg_text = []
-    if _directives.has_key(normname):
+    if normname in _directives:
         return _directives[normname], messages
     canonicalname = None
     try:
@@ -107,7 +113,7 @@ def directive(directive_name, language_module, document):
         # Error handling done by caller.
         return None, messages
     try:
-        module = __import__(modulename, globals(), locals())
+        module = __import__(modulename, globals(), locals(), level=1)
     except ImportError, detail:
         messages.append(document.reporter.error(
             'Error importing directive module "%s" (directive "%s"):\n%s'
@@ -204,6 +210,16 @@ def nonnegative_int(argument):
         raise ValueError('negative value; must be positive or zero')
     return value
 
+def percentage(argument):
+    """
+    Check for an integer percentage value with optional percent sign.
+    """
+    try:
+        argument = argument.rstrip(' %')
+    except AttributeError:
+        pass
+    return nonnegative_int(argument)
+
 length_units = ['em', 'ex', 'px', 'in', 'cm', 'mm', 'pt', 'pc']
 
 def get_measure(argument, units):
@@ -211,7 +227,7 @@ def get_measure(argument, units):
     Check for a positive argument of one of the units and return a
     normalized string of the form "<value><unit>" (without space in
     between).
-    
+
     To be called from directive option conversion functions.
     """
     match = re.match(r'^([0-9.]+) *(%s)$' % '|'.join(units), argument)
@@ -227,8 +243,26 @@ def get_measure(argument, units):
 def length_or_unitless(argument):
     return get_measure(argument, length_units + [''])
 
-def length_or_percentage_or_unitless(argument):
-    return get_measure(argument, length_units + ['%', ''])
+def length_or_percentage_or_unitless(argument, default=''):
+    """
+    Return normalized string of a length or percentage unit.
+
+    Add <default> if there is no unit. Raise ValueError if the argument is not
+    a positive measure of one of the valid CSS units (or without unit).
+
+    >>> length_or_percentage_or_unitless('3 pt')
+    '3pt'
+    >>> length_or_percentage_or_unitless('3%', 'em')
+    '3%'
+    >>> length_or_percentage_or_unitless('3')
+    '3'
+    >>> length_or_percentage_or_unitless('3', 'px')
+    '3px'
+    """
+    try:
+        return get_measure(argument, length_units + ['%'])
+    except ValueError:
+        return get_measure(argument, ['']) + default
 
 def class_option(argument):
     """
